<?php


// GETIMAGE FUNCTION ////////////////////////////
function getImage( $type, $string ) {
    // This function is the exact same as the imagegen.sc but
    // shortcodes can not be used for news titles so that is
    // where this comes in.
    
    // SEE "USER CONFIGURATION" SECTION BELOW FOR THE CHANGES YOU NEED TO MAKE
    
    // During one of the many rewrites of this script I found a script that did
    // something similar.  I like the way it did some things and encorporated parts
    // of it.  That script was by Stewart Rosenberger (http://www.stewartspeak.com/headings/
    
    // Directory where we are going to store are images. (chmod 777)
    $imageGenDir = THEME . "imagegen/generated_images/";
    
    // Directory where we can find the fonts.
    $fontDir = THEME . "imagegen/fonts/";

    // CLEANUP STRING /////////////////////////////////////////////////////////////
    $string = strip_tags($string);

    
    // CHECKS AND BALANCES ////////////////////////////////////////////////////////
    // Check for GD support
    if( !function_exists( 'ImageCreate' ) ) {
        return "ERROR: GD not supported.";
    }
    
    // Check for a type    
    if( $type == "" ) {
        return "IMAGEGEN:ERROR: Type was not specified";
    }
    
    // Check for a string
    if( $string == "" ) {
        return "IMAGEGEN:ERROR: String was not specified";
    }
    
    
    // USER CONFIGURATION /////////////////////////////////////////////////////////
    // You can modify the section below to set up your own types.  Types were the
    // easiest way to simplify this function/shortcode, as I did not want to have
    // to pass in all kinds of parameters but yet wanted flexability to generate
    // diferent types of images.  So all you need to pass in is a type and the
    // string.
        
    // Explanation of custom variables:
    //    $type            --> This is what you will pass in when you use IMAGEGEN (example: {IMAGEGEN=menu+My String} $type will be set to menu)
    //    $fileType        --> Either "jpg", "gif" or "png" (example: "gif")
    //    $font['font']    --> The name of the Ture Type Font to use.  NOTE: font must be located in e107_themes/YOUR_THEME_DIR/imagegen/fonts. (example: "Blackcha.TTF")
    //    $font['size']    --> The size of the font (example: 18)
    //    $font['color']   --> The color to use for the font in RGB hex format. (example: "#fofod9")
    //    $font['bground'] --> The color to use for the background in RGB hex format. (example: "#fofod9")
    //    $x_position      --> This is the position on the X axis where it starts writing.  (example: 0)
    //    $y_position      --> This is the position on the Y axis where it starts writing.  (example: 16) NOTE: usually safest to put the same number you did for size.
    //    $x_padding       --> This is a little extra to pad the image width with.  Sometimes fonts don't report their size correctly. (example: 5)
    //    $y_padding       --> This is a little extra to pad the image height with.  Sometimes fonts don't report their size correctly. (example: 0)
    // To add your own:
    //    Just add in a new elseif condition and give it a new type name.  Modify variables as needed.
    //
    // NOTE: For the most part $y_position should match $font['size'] but there
    // are exceptions to every rule.
    if( $type == 'carbon' ) {
        $fileType        = "gif";
        $font['font']    = "Carbonized_Timber.ttf";
        $font['size']    = 14;
        $font['color']   = "#000000";
        $font['bground'] = "#FFFFE6";
        $x_position      = 0;
        $y_position      = 14;
        $x_padding       = 5;
        $y_padding       = 0;
    } elseif( $type == 'news' ) {
        $fileType        = "gif";
        $font['font']    = "ENGLISHT.TTF";
        $font['size']    = 18;
        $font['color']   = "#000000";
        $font['bground'] = "#f0f0d9";
        $x_position      = 0;
        $y_position      = 16;
        $x_padding       = 5;
        $y_padding       = 0;
    } elseif( $type == 'sitetag' ) {
        $fileType        = "jpg";
        $font['font']    = "aftermat.ttf";
        $font['size']    = 18;
        $font['color']   = "#f0f0d9";
        $font['bground'] = "#840e0e";
        $x_position      = 0;
        $y_position      = 16;
        $x_padding       = 5;
        $y_padding       = 0;
    } elseif( $type == 'typewriter' ) {
        $fileType        = "gif";
        $font['font']    = "Typewriter-Regular.ttf";
        $font['size']    = 10;
        $font['color']   = "#0009";
        $font['bground'] = "#f0f0d9";
        $x_position      = 0;
        $y_position      = 10;
        $x_padding       = 5;
        $y_padding       = 0;
    } else {
        return "IMAGEGEN:ERROR type not defined: $type";
    }
    
    
    //I WOULDN'T MODIFY ANYTHING BELOW THIS LINE UNLESS YOU KNOW WHAT YOU'RE DOING.
    ///////////////////////////////////////////////////////////////////////////////
    
    // SETUP MORE VARIABLES ///////////////////////////////////////////////////////
    $font['font'] = $fontDir . $font['font'];
    $font['color'] = hexToRGB( $font['color'] );
    $font['bground'] = hexToRGB( $font['bground'] );
    
    
    // MORE CHECKS AND BALANCES ///////////////////////////////////////////////////
    // Check for specified font
    if( !is_readable( $font['font'] ) ) {
        return "IMAGEGEN:ERROR: The server is missing the specified font: " . $font['font'];
    }
    
    
    // If Magic Quotes is enabled then remove slashes
    if( get_magic_quotes_gpc () ) {
        $string = stripslashes($string) ;
    }
    
    
    // GENERATE IMAGE NAME ////////////////////////////////////////////////////////
    // Generate Image Name $type_$string.$fileType (example: menu_chatbox.jpg)
    $name = $string;
    // Strip special tags
    $name = strip_tags( $name );
    // Remove any character that is not a A-Z, a-z, 0-9 or _
    $imageName = preg_replace( "(\W)", '', $name );
    // Put it all together
    $imageName = "${imageGenDir}${type}_${imageName}.$fileType";
    
    
    // THE FUN STUFF //////////////////////////////////////////////////////////////    
    // Check to see if we need to create the image or did we do it already.
    if( file_exists( $imageName ) ) {
    
        // File currently exists so use it
        $imageInfo[IMAGE] = $imageName;
        list( $imageInfo['width'], $imageInfo['height'] ) = getimagesize( $imageInfo[IMAGE] );
    
    } else {
    
        // File does not currently exist so lets get to work
    
        // We need to calculate the dip (pixels below the baseline).
        $testString = 'abcdefghijklmnopqrstuvwxyz' . 
                      'ABCDEFGHIJKLMNOPQRSTUVWXYZ' .
                      '1234567890' .
                      '!@#$%^&*()\'"\\/;.,`~<>[]{}-+_-=' ;
        $testBox = imagettfbbox( $font['size'], 0, $font['font'], $testString );
        $dip = $testBox[1];
    
        // How big will the image need to be 
        $bbox = imagettfbbox( $font['size'], 0, $font['font'], $string );
        $imageInfo['width'] = abs( $bbox[2] - $bbox[0] ) + $x_padding; // width
        $imageInfo['height'] = abs( $bbox[7] - $dip ) + $y_padding; // height
    
        // Red Warrior Needs Food Badly
        // Green Wizzard Has Shot The Food
        // Red Warrior Is About To Die
        // :)  OK so I am tired :P
    
        if( $fileType == 'jpg' ) {
            $image  = imageCreateTrueColor( $imageInfo['width'], $imageInfo['height'] ); // Create a blank image
    
            imageAlphaBlending( $image, TRUE );
            imageSaveAlpha( $image, TRUE );
    
            // Set Some Colors
            $bgColor = imageColorAllocate( $image, $font['bground']['red'], $font['bground']['green'], $font['bground']['blue']);
            $textColor = ImageColorAllocate( $image, $font['color']['red'], $font['color']['green'], $font['color']['blue'] );
    
            imageFill( $image, 0, 0, $bgColor );    
    
            // Write on to the image
            imagettftext( $image, $font['size'], 0, $x_position, $y_position, $textColor, $font['font'], $string );
    
            imagejpeg($image, $imageName, 75);
        }elseif( $fileType == 'png' or $fileType == 'gif' ) {
            
            $image  = imageCreate( $imageInfo['width'], $imageInfo['height'] ); // Create a blank image
            if( !$image ) {
                return "IMAGEGEN:ERROR: unable to create the image: $imageName";
            }
    
            // Set Some Colors
            $bgColor = imageColorAllocate( $image, $font['bground']['red'], $font['bground']['green'], $font['bground']['blue']);
            $textColor = ImageColorAllocate( $image, $font['color']['red'], $font['color']['green'], $font['color']['blue'] );
    
            // Write on to the image
            imagettftext( $image, $font['size'], 0, $x_position, $y_position, $textColor, $font['font'], $string );
    
            ImageColorTransparent( $image, $bgColor );
    
            if( $fileType == 'png' ) {
                imagepng($image, $imageName);
            }else{
                imagegif($image, $imageName);
            }
        }
    }
    return "<img src='$imageName' width='$imageInfo[width]' height='$imageInfo[height]' alt='' style='display: block;' />";
}



// HEXTORGB FUNCTION ////////////////////////////
function hexToRGB( $hex ) {
    // Decode an HTML hex-code into an array of R,G, and B values.
    // Most of this function was taken from a sample script
    // Dynamic Heading Generator
    // By Stewart Rosenberger
    // http://www.stewartspeak.com/headings/    

    // remove '#'
    if( substr( $hex, 0, 1 ) == '#' ) {
        $hex = substr( $hex, 1 );
    }

    // expand short form ('fff') color
    if( strlen( $hex ) == 3 ) {
        $hex = substr($hex, 0, 1) . substr($hex, 0, 1) .
               substr($hex, 1, 1) . substr($hex, 1, 1) .
               substr($hex, 2, 1) . substr($hex, 2, 1) ;
    }

    // convert the hex into decimal
    if( strlen( $hex ) != 6 ) {
        // Not the correct format send back BLACK
        $rgb['red'] =   0;
        $rgb['green'] = 0;
        $rgb['blue'] =  0;
    }else{
        $rgb['red'] =   hexdec( substr( $hex, 0, 2 ) );
        $rgb['green'] = hexdec( substr( $hex, 2, 2 ) );
        $rgb['blue'] =  hexdec( substr( $hex, 4, 2 ) );
    }

    return $rgb ;
}

?>
